@@ -10,6 +10,7 @@
 #define SOCKERR WSAGetLastError()
 #define SOCKET_EINTR WSAEINTR
 #else
+#include <errno.h>
 #define SOCKERR errno
 #define SOCKET_EINTR EINTR
 #endif
@@ -469,11 +470,16 @@ bool PqResultImpl::step_run() {
   if (!data_ready_) {
     LOG_VERBOSE;
 
-    if (!wait_for_data()) {
-      pConnPtr_->cancel_query();
-    }
+    bool proceed = wait_for_data();
 
     data_ready_ = true;
+
+    if (!proceed) {
+      pConnPtr_->cancel_query();
+      complete_ = TRUE;
+      stop("Interrupted.");
+    }
+
     need_cache_reset = true;
   }
 
@@ -490,6 +496,8 @@ bool PqResultImpl::step_run() {
   }
 
   if (pRes_ == NULL) {
+    LOG_VERBOSE;
+
     complete_ = true;
     return false;
   }
@@ -580,6 +588,16 @@ bool PqResultImpl::wait_for_data() {
   if (!pConnPtr_->is_check_interrupts())
     return true;
 
+  // update db connection state using data available on the socket
+  if (!PQconsumeInput(pConn_)) {
+    stop("Failed to consume input from the server");
+  }
+
+  // check if PQgetResult will block before waiting
+  if (!PQisBusy(pConn_)) {
+    return true;
+  }
+
   int socket, ret;
   fd_set input;
   FD_ZERO(&input);
@@ -592,7 +610,7 @@ bool PqResultImpl::wait_for_data() {
   do {
     LOG_DEBUG;
 
-    // wait for any traffic on the db connection socket but no longer then 1s
+    // wait for any traffic on the db connection socket but no longer than 1s
     timeval timeout = {0, 0};
     timeout.tv_sec = 1;
     FD_SET(socket, &input);
@@ -608,6 +626,7 @@ bool PqResultImpl::wait_for_data() {
         checkUserInterrupt();
       }
       catch (...) {
+        LOG_DEBUG;
         return false;
       }
     } else if (ret < 0) {
